//=====================================================================================================================
/// @file		main.c
///
/// @brief		Main Program
///
/// @version	V1.0
///
/// @date		01.08.2020
///
/// @author		Philipp Holle
//=====================================================================================================================
//---------------------------------------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------------------------------------
#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include <avr/io.h>
#include "atmega328p.h"
#include "SPI_ATmega328p.h"
#include "RFM95W.h"
#include "ph_auxiliaries.h"
#include "lorawan.h"
#include "UART_atmega328p.h"
#include "I2C_atmega328p.h"

#include "STACKTEST.h"
//---------------------------------------------------------------------------------------------------------------------
// Macros
//---------------------------------------------------------------------------------------------------------------------
#define PRINT_LED(a)		PORTC = (a & ~(3<<6)); PORTD = (((a>>6)&3)<<3); //For Prototype Only

#define	CLOCKFREQ			16			//Frequency of Clocksource of MCU in MHz (post pre-scaler) // 1-16 MHz

#define CARRIER_FREQ		868100000	//Carrier Frequency in Hz				//862MHz to 1020MHz
#define BANDWIDTH			7			//Bandwidth of Chirp Spread Spectrum	//0=7.8kHz, 1=10.4kHz, 2=15.6kHz, 3=20.8kHz
																				//4=31.25kHz, 5=41.7kHz, 6=62.5kHz
																				//7=125kHz, 8=250kHz, 9=500kHz
#define CODING_RATE			1			//Error Coding Rate 					//1=4/5, 2=4/6, 3=4/7, 4=4/8	//4/5 = LoRaWAN standard
#define SPREAD_FACTOR		7			//Spreading Factor (Chips/Symbol)		//7=128, 8=256, 9=512, 10=1024, 11=2048,
																				//12=4096
#define CRC_ON				1			//CRC									//0=disable, 1=enable //LoRaWAN standard!
#define PREAMBLE_LEN		8			//Length of Preamble					// 8 = LoRaWAN standard
#define SYNC_WORD			0x34		//Sync Word											
							
//---------------------------------------------------------------------------------------------------------------------
// Global Variables
//---------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------
// Prototypes
//---------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------
// Functions
//---------------------------------------------------------------------------------------------------------------------

//=== Function: main ==================================================================================================
///
/// @brief	Main Program
///
/// @param	void
///
/// @return	void
//=====================================================================================================================
int main(void)
{
	//*************************************************************************
	// Variables
	//*************************************************************************
	uint8_t i;
	uint8_t test[127];
	uint8_t length;
	//*************************************************************************
	// Inits
	//*************************************************************************
	//Initialize Atmega328p MCU
	Atmega328pInit(CLOCKFREQ);
	//Initialize SPI interface
	SpiInitMaster();
	//Initialize UART interface
	UartInit(CLOCKFREQ, 9600);
	//Initialize I2C interface
	I2cInit(CLOCKFREQ, 50);
	//Initialize RFM95 LoRa Transceiver with initial transmission parameters
	Rfm95Init(CARRIER_FREQ, BANDWIDTH, CODING_RATE, SPREAD_FACTOR, CRC_ON, PREAMBLE_LEN, SYNC_WORD);
	
	
	//Set Port PC0-PC5 to Output
	//DDRC |= (1<<DDC0) | (1<<DDC1) | (1<<DDC2) | (1<<DDC3) | (1<<DDC4) | (1<<DDC5);
	//Set PD3 & PD4 to Output
	//DDRD |= (1<<DDD3) | (1<<DDD4);
	//*************************************************************************
	// While Loop
	//*************************************************************************
	while (1)
	{
		
		
		for (i=0;i<5;i++)
		{
			Atmega328pStopWatchStart(1000);
			while(!Atmega328pStopWatchCheck());
		}
		test[0] = 0x01;
		test[1] = 0x02;
		test[2] = 0x03;
		UartPrintLine("I2C init",0,0);
		UartPrintNewLine;
		
		i2cTxMaster(&test[0], 3, 0x01);
		
		/*
		sei();
		while(1)
		{
			//test[0] = (TWBR&0xFF);
			//UartPrintLine("TWBR: ", &test[0],1);
			//UartPrintNewLine();
			
			if (payloadPendingUp)
			{
				UartPrintLine("I2C receive: ", &payloadPendingUpContent[0], payloadPendingUpLength);
				UartPrintNewLine();
				payloadPendingUp = false;
			}
		}
		*/
		
		/*
		devNonce[0] = 0xA4;
		for (i=0;i<10;i++)
		{
			Atmega328pStopWatchStart(1000);
			while(!Atmega328pStopWatchCheck());
		}
		LorawanStartSession();
		*/
		
	}
	
	
	
}

ISR(TWI_vect)
{
	uint8_t i;
	i = TWSR;
	
	UartPrintLine("I2C IR entered",0,0);
	UartPrintNewLine();
	
	//Disable TWI interrupt
	TWCR &= !(1<<TWIE); 
	if (((TWSR & 0xF8) == 0x60) || ((TWSR & 0xF8) == 0x70))
	{
		
		UartPrintLine("Own address or gc received",0,0);
		UartPrintNewLine();
		
		payloadPendingUpLength=0;
		//Clear TWINT flag
		TWCR |= (1<<TWINT)|(1<<TWEN);
		//Wait for TWINT flag
		while(!(TWCR & (1<<TWINT)));
		while ((TWSR & 0xF8) != 0xA0)
		{
			
			UartPrintLine("Receive Byte",0,0);
			UartPrintNewLine();
			
			payloadPendingUpContent[payloadPendingUpLength] = TWDR;
			payloadPendingUpLength++;
			//Clear TWINT flag
			TWCR |= (1<<TWINT)|(1<<TWEN);
			//Wait for TWINT flag
			while(!(TWCR & (1<<TWINT)));
		}
		
		UartPrintLine("I2C STOP",0,0);
		UartPrintNewLine();
		
	}
	
	UartPrintLine("SC: ", &i,1);
	UartPrintNewLine();
	
	UartPrintLine("Exit IR loop",0,0);
	UartPrintNewLine();
	
	payloadPendingUp = true;
	//Enable TWI interrupt
	TWCR |= (1<<TWIE);
}